/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */

#ifndef _FFT_ANALYSIS_H_
#define _FFT_ANALYSIS_H_

#include <vector>
#include <complex>
#include <fftw3.h>
#include "window_function.h"

double convert_log_scale(double x, double pow = 2);

class FftComplex {
public:
    inline const double& operator[](size_t i) const {
        return data_[i];
    }
    inline double& operator[](size_t i) {
        return data_[i];
    }
    double abs() const;

    FftComplex(double real = 0, double imag = 0);

private:
    double data_[2];
};

class FftAnalysis {
public:
    FftAnalysis(size_t window_size);
    FftAnalysis(const WindowFunction& window);
    ~FftAnalysis() throw();
    void initialize();
    void execute_fourier_transform(std::vector<double>::const_iterator first,
                                   std::vector<FftComplex>& spectrum);
    void execute_fourier_transform(const std::vector<double>& signal,
                                   std::vector<FftComplex>& spectrum);

    void execute_DCTransform(std::vector<double>::const_iterator first,
                             std::vector<double>& spectrum);
    void execute_DCTransform(const std::vector<double>& signal,
                             std::vector<double>& spectrum);

    void execute_inverse_transform(const std::vector<FftComplex>& spectrum,
                                   std::vector<double>& signal, bool windowed = true);

private:
    fftw_complex* alloc_fft_location(size_t size) const;
    void free_fft_location(fftw_complex* ptr) const;

    const WindowFunction& window_;
    size_t win_size_;
    fftw_plan p_c2r_, p_r2c_, p_r2r_;
    double *r_data_, *r_out_data_;
    fftw_complex *c_data_;
};

#endif
